# -*- coding: utf-8 -*-
from chibi.config import configuration, basic_config


basic_config()

configuration.loggers[ 'vcr.stub' ].level = 'INFO'
#configuration.loggers[ 'vcr.cassette' ].level = 'WARNING'
configuration.loggers[ 'vcr' ].level = 'WARNING'



import logging
import time

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver.common.by import By
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import undetected_chromedriver as uc

from .exceptions import Cannot_pass_cloud_flare, Cannot_login
from .regex import re_show, re_follow, re_pending, re_read
from chibi_dl.site.base.site import Site


logger = logging.getLogger( "chibi_dl.sites.tmo_fans" )


class TMO_fans( Site ):
    def __init__( self, *args, url=None, user=None, password=None, **kw ):
        if url is None:
            url = 'https://tmofans.com/login'
        super().__init__( url, *args, **kw )
        self.series = []
        self.user = user
        self.password = password
        self._login_ok = False

    def append( self, url ):
        from .serie import Serie
        #self.cross_cloud_flare()
        if re_show.match( url ):
            self.series.append( Serie( url=url, parent=self ) )
        elif re_follow.match( url ):
            self.login()
            for l in self.get_all_follow():
                self.append( l )
        elif re_pending.match( url ):
            self.login()
            for l in self.get_all_pending():
                self.append( l )
        elif re_read.match( url ):
            self.login()
            for l in self.get_all_read():
                self.append( l )
        else:
            logger.error(
                "la url {} no se pudo identificar como serie".format( url ) )

    def cross_cloud_flare( self, delay=2 ):
        super().cross_cloud_flare( delay=delay )
        self.browser.close()

    @property
    def soup( self ):
        if not self.cloud_flare_passed:
            self.cross_cloud_flare()
        return super().soup

    def login( self ):
        if self._login_ok:
            return
        if not self.cloud_flare_passed:
            self.cross_cloud_flare()

        email = self.firefox.find_element( By.ID, "email" )
        password = self.firefox.find_element( By.ID, "password" )
        submit = self.firefox.find_element(
            By.XPATH,
            "/html/body/div[1]/main/div/div/div/div[1]/form/div[4]"
            "/div[1]/button" )

        email.send_keys( self.user )
        password.send_keys( self.password )

        submit.click()
        logger.info( "esperando 10 segundos a que termine el login" )
        time.sleep( 10 )
        if self.browser.current_url == self.url:
            raise Cannot_login
        else:
            self._login_ok = True
            self.cookies = self.firefox.get_cookies()
            self.user_agent = self.firefox.execute_script(
                "return navigator.userAgent;" )

        """
        page = self.get( self.url )
        soup = BeautifulSoup( page.content, 'html.parser' )
        form = soup.find( "form", { "class": "form-horizontal" } )
        token = form.find( "input", dict( name="_token" ) )[ "value" ]

        payload = dict(
            email=self.user, password=self.password, remember="on",
            _token=token )

        response = self.session.post(
            self.url, data=payload, headers={
                "Referer": str( self.url ),
                "Content-type": "application/x-www-form-urlencoded" } )

        if response.ok and response.url == self.url:
            raise Cannot_login
        else:
            self._login_ok = True
        """

    def get_all_follow( self ):
        page_number = 0
        url = "https://tmofans.com/profile/follow"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_all_pending( self ):
        page_number = 0
        url = "https://tmofans.com/profile/pending"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_all_read( self ):
        page_number = 0
        url = "https://tmofans.com/profile/read"
        while( True ):
            page_number += 1
            page = self.get( url=url, params={ "page": page_number } )
            soup = BeautifulSoup( page.content, 'html.parser' )

            links = self.get_manga_links( soup )
            if not links:
                return
            for l in links:
                yield l

    def get_manga_links( self, soup ):
        result = []
        for a in soup.find_all( "a" ):
            if "library/" in a[ "href" ]:
                result.append( a[ "href" ].strip() )
        return result

    def i_can_proccess_this( self, url ):
        regex = ( re_show, re_pending, re_read, re_follow )
        return any( ( r.match( url ) for r in regex ) )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from unittest import TestCase, skip
from chibi.file import Chibi_path
from vcr_unittest import VCRTestCase
from chibi.file.temp import Chibi_temp_path
from chibi_dl_tmo.site import TMO_fans
from vcr.record_mode import RecordMode
import logging


class Test_tmofans:
    def _get_vcr_kwargs( self, **kw ):
        result = super()._get_vcr_kwargs( **kw )
        result[ 'ignore_localhost' ] = True
        result[ 'ignore_hosts' ] = [
            'chromedriver.storage.googleapis.com',
        ]
        #result[ 'record_mode' ] = RecordMode.NEW_EPISODES
        return result

    def test_the_series_should_find_his_own_name( self ):
        self.assertIsInstance( self.site.series[0].name, str )

    def test_have_a_list_of_episodes( self ):
        self.assertTrue( self.site.series[0].episodes )
        for episode in self.site.series[0].episodes:
            self.assertTrue( episode.title )
            self.assertTrue( episode.fansub )
            self.assertTrue( episode.url )
            self.assertIsInstance( float( episode.number ), float )

    def test_the_episode_should_have_a_list_of_images( self ):
        episode = self.site.series[0].episodes[0]
        self.assertTrue( episode.images_urls )

    def test_should_download_the_images_in_a_folder( self ):
        folder = Chibi_temp_path()
        episode = self.site.series[0].episodes[0]
        episode.download( folder )
        self.assertTrue( next( folder.ls() ) )
        for f in folder.ls():
            properties = f.open().properties
            self.assertGreater( properties.size, 1024 )

    @skip( 'esta mierda es muy lenta' )
    def test_should_can_compress_the_download_files( self ):
        folder = Chibi_temp_path()
        folder_compress = Chibi_temp_path()
        episode = self.site.series[0].episodes[0]
        episode.download( folder )
        result = episode.compress( folder_compress, folder )
        self.assertIsInstance( result, Chibi_path )
        self.assertTrue( result.exists )
        self.assertTrue( result.endswith( ".cbz" ) )
        self.assertIn( result, folder_compress )

    @skip( 'esta mierda es muy lenta' )
    def test_should_donwload_all_the_serie( self ):
        folder = Chibi_temp_path()
        serie = self.site.series[0]
        serie.download( folder )
        download_files = list( next( folder.ls() ).ls() )
        self.assertEqual( len( serie.episodes ), len( download_files ) )


@skip( "quitaron el link" )
class Test_spy_family( Test_tmofans, VCRTestCase ):
    def setUp( self ):
        super().setUp()
        self.site = TMO_fans()
        self.site.append(
            'https://lectortmo.com/library/manga/43882/spy-x-family' )

    def test_serie_should_be_the_url_of_the_spy_x_family( self ):
        self.assertEqual(
            self.site.series[0].url,
            'https://lectortmo.com/library/manga/43882/spy-x-family' )


class Test_king_from_hell( Test_tmofans, VCRTestCase ):
    def setUp( self ):
        super().setUp()
        self.site = TMO_fans()
        self.site.append(
            'https://visortmo.com/library/manga/79656/king-from-hell' )

    def test_serie_should_be_the_url_of_the_spy_x_family( self ):
        self.assertEqual(
            self.site.series[0].url,
            'https://visortmo.com/library/manga/79656/king-from-hell' )


class Test_tmp_basic( TestCase ):
    def setUp( self ):
        super().setUp()
        self.site = TMO_fans()

    def test_on_init_should_not_have_browser( self ):
        self.assertFalse( self.site._has_browser_init )

    def test_should_no_open_the_browser_when_do_the_append( self ):
        self.assertFalse( self.site.is_browser_open )
        self.site.append( 'https://lectortmo.com/' )
        self.assertFalse( self.site.is_browser_open )



import logging
import re
import shutil
import time

from bs4 import BeautifulSoup
from chibi.file import Chibi_path
from chibi_requests import Chibi_url
from chibi_dl.site.base.exceptions import Max_retries_reach
from chibi_dl_tmo.site import TMO_fans as Site


logger = logging.getLogger( "chibi_dl.sites.tmo_fans.episode" )


class Episode( Site ):
    def download( self, path ):
        logger.info(
            "iniciando la descarga de las {} imagenes del capitulo {}".format(
                len( self.images_urls ), self.number,
            ) )
        for image_name, url in self.enumerate_images_urls():
            referer = self.url + '/'
            image = self.get(
                url=url, headers={ 'Referer': str( referer ) },
                ignore_status_code=[ 403 ], verify=False )
            if not image.ok:
                logger.warning( "no se encontro una imagen" )
                continue
            full_path = path + image_name
            f = full_path.open()
            logger.debug( "descargando {}".format( url ) )
            f.write( image.content )
            logger.debug( f"imagen {f} guardada" )

    def compress( self, path_ouput, path_input, format="zip" ):
        logger.info( "comprimiendo capitulo usando {}".format( format ) )
        file_name = str( path_ouput + self.number )
        result = Chibi_path( shutil.make_archive(
            file_name, format, str( path_input ) ) )
        expected = result.replace_extensions( "cbz" )
        result.move( expected )
        return expected

    @property
    def file_name( self ):
        return "{}.{}".format( self.number, "cbz" )

    @property
    def number( self ):
        re_float = re.compile( r"[-+]?([0-9]*\.[0-9]+|[0-9]+)" )
        return re_float.findall( self.title )[0]

    @property
    def images_urls( self ):
        try:
            return self._images_urls
        except AttributeError:
            self.load_soup()
            return self._images_urls

    def load( self ):
        response = self.get(
            url=self.url, headers={ "Referer": str( self.parent.url ) } )
        self._response = response

    def url_need_to_redirect( self, url ):
        return 'view_uploads' in url

    def load_soup( self, delay=0, retries=0, max_retries=5, ):
        if retries > max_retries:
            logger.exception( "maximo numero de reintentos para {url}" )
            raise Max_retries_reach( self.url )
        if delay > 0:
            time.sleep( delay )

        self.load()

        if self.url_need_to_redirect( self._response._response.url ):
            regex = r"uniqid:'(.*)'"
            match = re.search( regex, self.soup.select( "script" )[0].text )
            if match:
                episode_id = match.groups()[0]
                self.url = (
                    Chibi_url( 'https://visortmo.com/viewer' ) + episode_id )
                self.url = self.url + 'cascade'
                self.load()
        first_link = self.soup.select_one( 'a' )
        if first_link and first_link.get( 'href' ) != 'https://visortmo.com':
            import pdb
            pdb.set_trace()
            logger.info(
                f'load_soup el primer link no tiene '
                f'la pagina de tmo {first_link}' )
            parts = Chibi_url( self._response._response.url ).path.rsplit( '/', 2 )[-2:]
            url = Chibi_url( 'https://visortmo.com/viewer' ) + parts[0]
            url = url + 'cascade'
            logger.info( f"contrullendo la url {url}" )
            self.load_soup_cascade( url )
            return

        if 'cascade' not in self._response._response.url:
            try:
                cascade_url = self.soup.select_one(
                    'a.nav-link[title="Cascada"]' ).get( 'href' )
            except:
                lines = self.soup.script.text.split('\n')
                lines = list( filter( bool, lines ) )
                dict_pk = lines[0].split( '=' )[1]
                pk = dict_pk.split( ',' )[0].split( ':' )[1]
                pk = pk.replace( "'", "" ).strip()
                cascade_url = (
                    Chibi_url( 'https://visortmo.com/viewer' ) + pk ) + 'cascade'
        else:
            cascade_url = self._response._response.url
        self.load_soup_cascade( cascade_url )

    def load_soup_cascade( self, url, delay=0, retries=0, max_retries=5, ):
        if retries > max_retries:
            logger.exception( "maximo numero de reintentos para {url}" )
            raise Max_retries_reach( self, url )
        if delay > 0:
            time.sleep( delay )

        cascade = self.get( url=url )
        soup = BeautifulSoup( cascade.content, 'html.parser' )

        first_link = soup.select_one( 'a' ).get( 'href' )
        if first_link != 'https://visortmo.com':
            logger.info(
                f'load_soup_cascade el primer link no tiene '
                f'la pagina de tmo {first_link}' )
            parts = Chibi_url(
                cascade._response.url ).path.rsplit( '/', 2 )[-2:]
            url = Chibi_url( 'https://visortmo.com/viewer' ) + parts[0]
            url = url + 'cascade'
            logger.info( f"contrullendo la url {url}" )
            self.load_soup_cascade( url  )
            return
            self.load_soup( delay=10, retries=retries + 1 )
        container = soup.find(
            "div", { "class", "viewer-container container" } )
        if not container:
            self.load_soup( url, delay=10, retries=retries + 1 )
        images = container.find_all( "img" )
        self._images_urls = [ img.get( "data-src" ) for img in images ]

    def enumerate_images_urls( self ):
        for i, url in enumerate( self.images_urls ):
            ext = url.rsplit( '.', 1 )[-1]
            yield "{}.{}".format( i, ext ), url



# -*- coding: utf-8 -*-
"""Console script for chibi_dl_tmo."""
import argparse
import sys
import json
import sys
import logging
import random
from argparse import ArgumentParser
import urllib3

from chibi.file import Chibi_path
from chibi.config import basic_config, load as load_config
from chibi.config import default_file_load, configuration

from chibi_dl_tmo import TMO_fans

default_file_load( 'chibi_dl_tmo.py', touch=True )
logger_formarter = '%(levelname)s %(name)s %(asctime)s %(message)s'


parser = ArgumentParser(
    description="descarga mangas", fromfile_prefix_chars='@'
)

parser.add_argument(
    "sites", nargs='+', metavar="site",
    help="urls de las series que se quieren descargar" )

parser.add_argument(
    "--only_print", dest="only_print", action="store_true",
    help="define si silo va a imprimir la lista de links o episodios"
)

parser.add_argument(
    "--only_metadata", dest="only_metadata", action="store_true",
    help="se define si solo se queire recolectar los datos y no descargar"
)

parser.add_argument(
    "--only_links", dest="only_print_links", action="store_true",
    help="si se usa solo imprimira las urls"
)

parser.add_argument(
    "--user", '-u', dest="user", default="",
    help="usuario del sitio" )

parser.add_argument(
    "--password", '-p', dest="password", default="",
    help="contrasenna del sitio" )

parser.add_argument(
    "--log_level", dest="log_level", default="INFO",
    help="nivel de log",
)

default_download_path=configuration.chibi_dl_tmo.get( 'download_path' )
if default_download_path:
    parser.add_argument(
        "-o", "--output", type=Chibi_path, dest="download_path",
        default=default_download_path,
        help="ruta donde se guardara el video o manga" )
else:
    parser.add_argument(
        "-o", "--output", type=Chibi_path, dest="download_path", required=True,
        help="ruta donde se guardara el video o manga" )

parser.add_argument(
    "-config_site", type=Chibi_path, dest="config_site",
    help="python, yaml o json archivo con el usuario y password de cada sitio"
)

parser.add_argument(
    "--amount", type=int, dest="amount", default=None,
    help="cantidad de episodios que descargara"
)


def main():
    """Console script for chibi_dl_tmo."""
    # tmo falla de vez en cuando con el https o siempre
    urllib3.disable_warnings( urllib3.exceptions.InsecureRequestWarning )
    args = parser.parse_args()
    args.download_path
    basic_config( args.log_level )

    tmo_fans = TMO_fans( user=args.user, password=args.password, )

    for site in args.sites:
        tmo_fans.append( site )

    if args.only_print:
        if args.only_print_links:
            for serie in tmo_fans.series:
                print( serie.url )
    else:
        for serie in tmo_fans.series:
            serie.download( args.download_path, amount=args.amount )
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import re


re_show = re.compile( r'https://(tmofans|lectortmo|visortmo).com/library/\w*/\d*/.*' )

re_follow = re.compile( r'https://(tmofans|lectortmo|visortmo).com/profile/follow' )
re_pending = re.compile( r'https://(tmofans|lectortmo|visortmo).com/profile/pending' )
re_read = re.compile( r'https://(tmofans|lectortmo|visortmo).com/profile/read' )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ 'chibi-dl>=0.1.2', 'undetected-chromedriver>=3.5.2' ]

setup(
    author="dem4ply",
    author_email='dem4ply@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="command script for download mangas from lectortmo.com or visortmo.com",
    entry_points={
        'console_scripts': [
            'chibi_dl_tmo=chibi_dl_tmo.cli:main',
        ],
    },
    install_requires=requirements,
    license="WTFPL",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='chibi_dl_tmo',
    name='chibi_dl_tmo',
    packages=find_packages(include=['chibi_dl_tmo', 'chibi_dl_tmo.*']),
    url='https://github.com/dem4ply/chibi_dl_tmo',
    version='1.0.0',
    zip_safe=False,
)



import copy
import itertools
import logging
import time

from bs4 import BeautifulSoup
from chibi.file.temp import Chibi_temp_path
from chibi.atlas import Chibi_atlas

from .episode import Episode
from chibi_dl_tmo.site import TMO_fans as Site


logger = logging.getLogger( "chibi_dl.sites.tmo_fans.serie" )


class Serie( Site ):
    def download( self, path, amount=None ):
        serie_path = ( path + self.name ).made_safe()
        serie_path.mkdir()

        meta = serie_path + "metadata.yml"
        meta.open().write( self.metadata )
        basura, ext = self.cover_url.base_name.rsplit( '.', 1 )
        self.cover_url.download( serie_path + f'cover.{ext}' )

        logger.info( "iniciando descarga de la serie '{}' de {}".format(
            self.name, self.url ) )
        if amount:
            episodes = itertools.islice( self.episodes, amount )
        else:
            episodes = self.episodes
        for episode in episodes:
            episode_path = serie_path + episode.file_name
            if ( episode_path.exists ):
                logger.info( (
                    "ignorando el episodio {} se encontro "
                    "en el destino" ).format( episode.title ) )
                continue
            downlaod_folder = Chibi_temp_path()
            try:
                episode.download( downlaod_folder )
            except Exception as e:
                logger.exception(
                    "paso un problema cuando intento de "
                    "descargar el episodio" )
                print( e )
                #import pdb
                #pdb.post_mortem( e.__traceback__ )
                #import pdb
                #pdb.set_trace()
                continue
            path_episode = episode.compress( serie_path, downlaod_folder )
            logger.info( f"episodio descargado '{path_episode}'" )
            logger.info(
                'termino de descargar el capitulo esperando '
                '10 segundos' )
            time.sleep( 10 )
        logger.info(
            'termino de descargar los episodios esperando '
            '5 segundos' )
        time.sleep( 5 )

    @property
    def name( self ):
        return self.info.title

    @property
    def episodes( self ):
        return self.info.episodes

    @property
    def cover_url( self ):
        img = self.soup.select( "section" )[1].select_one( "img" )
        return self.build_url( img.get( 'src' ) )

    def parse_metadata( self ):
        meta = copy.copy( self.info )
        meta.url = str( self.url )
        meta.pop( 'episodes' )
        return meta

    def parse_info( self ):
        result = Chibi_atlas()
        try:
            result.title = "".join(
                self.soup.select( ".element-title.my-2" )[0].find_all(
                    text=True, recursive=False ) ).strip()
            result.episodes = self.load_episodes( self.soup )
            data = self.soup.select( "section" )[1]
            result.tags = list( c.text.lower() for c in data.select( "h6" ) )
            result.other_titles = list(
                c.text.lower() for c in data.select( "span.badge" ) )
            result.synopsis = data.select_one( 'p.element-description' ).text

            staff = self.soup.select( "div.card-body" )
            result.staff = []
            for s in staff:
                rol = s.b.text
                name = s.h5.text
                name = name.replace( ',', '' ).strip()
                result.staff.append( dict( rol=rol, name=name ) )
        except Exception as e:
            print( e )
            import pdb
            pdb.post_mortem( e.__traceback__ )
            import pdb
            pdb.set_trace()
            raise
        return result

    def load_episodes( self, soup ):
        episodes = []
        if "one_shot" in self.url:
            return self.load_one_shot( soup )
        else:
            chapter_container = soup.find(
                "div", { 'class': "card chapters" } )
            chapter_container_hidden = chapter_container.find(
                "div", id="chapters-collapsed" )

            if not chapter_container_hidden:
                chapters = chapter_container.ul.find_all(
                    "li", recursive=False )
            else:
                chapters = itertools.chain(
                    chapter_container.ul.find_all( "li", recursive=False ),
                    chapter_container_hidden.find_all( "li", recursive=False )
                )

            for chapter in chapters:
                links = chapter.select(
                    "div.card.chapter-list-element" )[0].find_all( 'a' )
                fansub = links[0].text
                title = chapter.find( "h4" ).a.text
                url = links[0].find_next(
                    "span", { "class": "fas fa-play fa-2x"}
                ).parent.get( 'href' )

                episodes.append(
                    Episode.from_site(
                        site=self, url=url, fansub=fansub, title=title ) )
        return episodes

    def load_one_shot( self, soup ):
        episodes = []
        chapters = soup.find( "div", { "class": "card chapter-list-element" } )
        chapters = chapters.ul.find_all( "li", recursive=False )
        for i, chapter in enumerate( chapters ):
            parts = chapter.div.find_all( "div", recursive=False )
            url = parts[-1].a.get( "href" ).strip()

            episodes.append(
                Episode.from_site(
                    site=self, url=url, fansub=parts[0].text.strip(),
                    title=str( i ) ) )
        return episodes
